function simulation=LMMwithonevariance(X,Z,SIG,lambda0,ML,nrep)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%This program was designed for simulations necessary for the article
%"Likelihood ratio tests in linear mixed models with one variance component"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Author:        Ciprian Crainiceanu
%Last revised:  January 16, 2004
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Goal:          Simulate the (R)LRT distribution in Theorem 3 for testing 
%            
%               \lambda=\lambda_0      vs.      \lambda \in \Lambda
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%The full model is
%
%               Y       =   X \beta + Z b +\epsilon
%               b       ~   N(0,\sigma^2_b \Sigma)
%               \epsilon~   N(0,\sigma^2_\epsilon I_n)
%
%Here           \lambda=\sigma^2_b/\sigma^2_\epsilon
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Input Parameters: All parameters are required. Order needs to be preserved 
%
%   X       =   Design matrix for fixed effects (including deterministic 
%               intercept)
%
%   Z       =   Design matrix for random effects 
%
%   SIG     =   Known correlation matrix of random effects (= \Sigma). For 
%               P-splines \Sigma=I_K. Here K is the number of columns of the 
%               Z matrix (number of random effects)
%
%   lambda0 =   True value of the ratio of random effects and error 
%               variances. This value has to be positive and can be zero
%
%   ML      =   Indicator of the type of criterion used. ML=1 corresponds  
%               to Maximum Likelihood and ML=0 corresponds to Restricted 
%               Maximum Likelihood
%
%   nrep    =   Number of thousands of simulations needed. E.g. nrep=2  
%               corresponds to 2,000 simulations
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Output parameters: the output is in the form of a Matlab structure
%
%distribution       =   The null finite sample distribution of the LRT or RLRT.
%                       This is a row vector of dimension 1000*nrep
%
%probability_at0    =   Finite sample probability at zero of the (R)LRT 
%                       distribution
%
%prob_smaller_lambda0=  Probability of underestimating \lambda_0 using either
%                       ML or REML criterion
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Functions called: simulatenull
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n=size(X,1);                         % Number of observations
p=size(X,2)-1;                       % Number of covariates (no intercept)

sSIG=sqrtm(SIG);                     % Principal square root of \Sigma

P0=eye(n)-X*inv(X'*X)*X';            % Projection matrix on           

mu=eig(sSIG*Z'*P0*Z*sSIG);           % Calculate the eigenvalues from Theorem 1
xi=eig(sSIG*Z'*Z*sSIG);


simulation=simulatenull(mu,xi,p,n,lambda0,ML,nrep);