function RLRTstatistic=maximizeRLRT(Y,X,Z,SIG,ML)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%This program was designed for simulations necessary for the article
%"Likelihood ratio tests in linear mixed models with one variance component"
%The program is provided as is with no implied warranties.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Author:        Ciprian Crainiceanu
%Last revised:  January 16, 2004
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%The full model is
%
%               Y       =   X \beta + Z b +\epsilon
%               b       ~   N(0,\sigma^2_b \Sigma)
%               \epsilon~   N(0,\sigma^2_\epsilon I_n)
%
%Here           \lambda=\sigma^2_b/\sigma^2_\epsilon
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Goal:          Calculate (R)LRT for testing 
%            
%               \lambda=0           vs.      \lambda >  0
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Input Parameters: All parameters are required. Order needs to be preserved 
%
%   Y       =   Column vector of responses
%
%   X       =   Design matrix for fixed effects
%
%   Z       =   Design matrix for random effects
%
%   SIG     =   Known correlation matrix of random effects
%
%   ML      =   Indicator of the type of criterion used. ML=1 corresponds  
%               to Maximum Likelihood and ML=0 corresponds to Restricted 
%               Maximum Likelihood
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Output parameters: the output is in the form of a Matlab structure
%
%RLRTgrid           =   (R)LRT statistic calculate on a 401-dimensional grid  
%                       of points, equally spaced on the log scale 
%
%RLRTvalue          =   Maximum of the (R)LRT statistic
%
%lambdamax          =   Value of \lambda where the maximum is attained
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Functions called: none
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


format long

lam=[0 logspace(-10,10,400)]';      %Define the grid of points for \lambda


n=size(Y,1);                        %Sample size                          
n0=n;
K=size(Z,2);                        %Number of random effects
S=eye(n)-X*inv(X'*X)*X';

a=Y'*S*Y;                           %Define important matrices. This ensures
b=Y'*S*Z;                           %that same calculations are not done 
c=Z'*S*Z*SIG;                       %than once
d=Z'*Z*SIG;



if (ML==0)
    n0=n-size(X,2);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Calculate the LRT or RLRT values on the grid of points for lambda
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


for i=1:size(lam,1)
    aux1(i)=n0*log(a-lam(i)*b*SIG*inv(eye(K)+lam(i)*c)*b');
    if (ML==1)
        aux2(i)=log(det(eye(K)+lam(i)*d));
    end
    if (ML==0)
        aux2(i)=log(det(eye(K)+lam(i)*c));
    end
    aux3(i)=aux1(i)+aux2(i);
end

RLRT=n0*log(a)-aux3;

[m,ind]=max(RLRT);

RLRTstatistic=struct('RLRTgrid',RLRT,'RLRTvalue',m,'lambdamax',lam(ind))
