function simulation=simulatenull(mu,xi,p,n,lambda0,ML,nrep)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%This program was designed for simulations necessary for the article
%"Likelihood ratio tests in linear mixed models with one variance component"
%The program is provided as is without any implied warranties.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Author:        Ciprian Crainiceanu
%Last revised:  May 29, 2004
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Goal:          Simulate the (R)LRT distribution in Theorem 3 for testing 
%            
%               \lambda=\lambda_0      vs.      \lambda \in \Lambda
%
%               For \lambda_0=0 one obtains the null distribution for 
%               testing that the variance of random effects is zero
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Input Parameters: All parameters are required. Order needs to be preserved 
%
%   mu      =   Vector of eigenvalues \mu_{s,n} described in Theorem 1
%               dimension is equal to the number of random effects
%
%   xi      =   Vector of eigenvalues \xi_{s,n} described in Theorem 1
%               dimension is equal to the number of random effects
%
%   p       =   Number of covariates (number of columns of the X matrix
%               minus one). E.g. for ANOVA 1, p=0
%
%   n       =   Total number of observation
%
%   lambda0 =   True value of the ratio of random effects and error 
%               variances. This value has to be positive and can be zero
%
%   ML      =   Indicator of the type of criterion used. ML=1 corresponds  
%               to Maximum Likelihood and ML=0 corresponds to Restricted 
%               Maximum Likelihood
%
%   nrep    =   number of thousands of simulations needed. E.g. nrep=2  
%               corresponds to 2,000 simulations
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Output parameters: the output is in the form of a Matlab structure
%
%distribution       =   The null finite sample distribution of the LRT or RLRT.
%                       This is a row vector of dimension 1000*nrep
%
%probability_at0    =   Finite sample probability at zero of the (R)LRT 
%                       distribution
%
%prob_smaller_lambda0=  Probability of underestimating \lambda_0 using either
%                       ML or REML criterion
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Functions called: none
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

format long
K=size(mu,1);           %Number of random effects
I=1000;                 %Number of simultaneous simulations in one iteration
nsize=I*nrep;           %Total number of simulations

n0=n;                   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Correcting values for the case when REML is used (ML=0)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (ML==0)
    xi=mu;
    n0=n-p-1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Setting up the grid of \lambda-points where the null distribution is 
%calculated. 
%
%If \lambda_0=0 then the grid is finer closer to 0 and coarser further away. 
%400 grid points equally spaced on the log scale
%
%If \lambda_0>0 a grid is constructed so that the grid is finer around 
%\lambda_0 and coarser further away
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (lambda0==0)
    lgrid=logspace(-7,7,400);
    lgrid=[0 lgrid];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%This part of the program was changed to allow for more \lambda points in
%the neighborhood of zero.
%
%Changed: May 29, 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (lambda0>0)
    lgrid_left_aux1=lambda0-logspace(-7,log10(lambda0),200);
    lgrid_left_aux2=logspace(-7,log10(lambda0),200);
    lgrid_left=[lgrid_left_aux1 lgrid_left_aux2];
    lgrid_left=sort(lgrid_left);    
    lgrid_right=lambda0+logspace(-7,7,200);
    lgrid=[lgrid_left lambda0 lgrid_right];
end


lgrid=lgrid'; G=size(lgrid,1);
distribution=[];
indices=[];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Using Matlab matrix capability to do 1000 simulations at once. 
%\Lambda_mu is a G*K matrix with the i-th line corresponding to the 
%i-th grid point. Same for the other matrices.
%These matrices are needed for simulation (they are deterministic)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%


for i=1:G
    for j=1:K
        Lambda_mu(i,j)=lgrid(i)*mu(j);
        Lambda0_mu(i,j)=lambda0*mu(j);
        Lambda_xi(i,j)=lgrid(i)*xi(j);
        Lambda0_xi(i,j)=lambda0*xi(j);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%sumlog:    Stores the second part of the distribution described in Theorem 3
%           It is a G*I matrix with identical columns and the i-th row 
%           corresponding to the i-th grid point
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


AUX=log((ones(G,K)+Lambda_xi)./(ones(G,K)+Lambda0_xi));
sumlog=(sum(AUX'))';
sumlog=repmat(sumlog,1,I);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%W  :   Stores the first K w_s with \chi^2_1 distributions for each simulation
%       It is a K*I matrix with the i-th column corresponding to the i-th 
%       simulation
%
%WW :   Stores the last part of the denominator D_n(.,.). This is a 
%       a random variable with a chi^2_{n-p-K-1} distribution.
%       It is a G*I matrix with identical rows
%
%N_lam_lam0 :   Stores the numerator N_n(\lambda,\lamda_0) for each value
%               of \lambda on the grid. It is a G*K matrix
%
%D_lam_lam0 :   Stores the denominator D_n(\lambda,\lamda_0) for each value
%               of \lambda on the grid. It is a G*K matrix
%
%RLRT       :   Stores the (R)LRT. It is a 1*I matrix
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


for t=1:nrep
    t
    W=chi2rnd(1,K,I);
    WW=chi2rnd(n-p-1-K,1,I);
    WW=repmat(WW,G,1);
    N_lam_lam0=((Lambda_mu-Lambda0_mu)./(ones(G,K)+Lambda_mu))*W;
    D_lam_lam0=((ones(G,K)+Lambda0_mu)./(ones(G,K)+Lambda_mu))*W+WW;
    RLRT=n0*log(ones(G,I)+N_lam_lam0./D_lam_lam0)-sumlog;
    
    [RLRT IND]=max(RLRT);
    distribution=[distribution RLRT];
    indices=[indices IND];
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%prob0                  :   Probability mass at zero of null finite sample 
%                           distribution of the (R)LRT
%
%prob_smaller_lambda0   :   Probability of underestimating \lambda_0 using 
%                           the (RE)ML criteria
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
prob0=sum(indices==1)/nsize;

prob_smaller_lambda0=prob0;
if (lambda0>0)
    prob_smaller_lambda0=sum(indices<401)/nsize;
end


simulation=struct('distribution',distribution,'probability_at0',prob0,...
    'prob_smaller_lambda0',prob_smaller_lambda0)
    
    
    
    
    
    
    
    
    
    
    
    
        
    
    
