function simulation=simulationsplines(x,degree,nknots,lambda0,ML,nrep)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%This program was designed for the article
%"Likelihood ratio tests in linear mixed models with one variance component"
%The program is provided as is without any implied warranties.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Author:        Ciprian Crainiceanu
%Last revised:  January 16, 2004
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Goal:          Simulate the (R)LRT distribution in Theorem 3 for testing 
%            
%               \lambda=\lambda_0      vs.      \lambda \in \Lambda
%
%This function is dedicated to testing in the penalized spline models. Knots 
%are chosen at the quantiles of the observed covariates. Alternative knot
%definitions could be used by simply erasing the knot definition line. 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%The full model is
%
%               Y       =   X \beta + Z b +\epsilon
%               b       ~   N(0,\sigma^2_b \Sigma)
%               \epsilon~   N(0,\sigma^2_\epsilon I_n)
%
%Here           \lambda=\sigma^2_b/\sigma^2_\epsilon
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Input Parameters: All parameters are required. Order needs to be preserved 
%
%   x       =   Column vector of covariates
%
%degree     =   Degree of the spline 
%
%nknots     =   Number of knots of the spline
%
%lambda0    =   True value of the ratio of random effects and error 
%               variances. This value has to be positive and can be zero
%
%   ML      =   Indicator of the type of criterion used. ML=1 corresponds  
%               to Maximum Likelihood and ML=0 corresponds to Restricted 
%               Maximum Likelihood
%
%   nrep    =   Number of thousands of simulations needed. E.g. nrep=2  
%               corresponds to 2,000 simulations
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Output parameters: the output is in the form of a Matlab structure
%
%distribution       =   The null finite sample distribution of the LRT or RLRT.
%                       This is a row vector of dimension 1000*nrep
%
%probability_at0    =   Finite sample probability at zero of the (R)LRT 
%                       distribution
%
%prob_smaller_lambda0=  Probability of underestimating \lambda_0 using either
%                       ML or REML criterion
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Functions called: LMMwithonevariance, quantileknots, powerbasis01
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


knots =quantileknots(x,nknots,0) ;              %Knots are defined
xm  = powerbasis01(x,degree,knots) ;            %The [X Z] matrix is defined
xm1 = xm(:,1:(degree+1)) ;                      %The X matrix is defined
xm2 = xm(:,(2+degree):size(xm,2)) ;             %The Z matrix is defined


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%The identity matrix is used as penalty. Other penalty matrices could be 
%defined here. A list of options could be implemented.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


SIG=eye(size(xm2,2));

simulation=LMMwithonevariance(xm1,xm2,SIG,lambda0,ML,nrep);


